<?php
# -- BEGIN LICENSE BLOCK ----------------------------------
#
# This file is part of ePUBmaker, a plugin for Dotclear 2.
# 
# Copyright (c) 2012 Osku and contributors
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
#
# -- END LICENSE BLOCK ------------------------------------
if (!defined('DC_RC_PATH')) { return; }

class ePUBmaker
{
	public $tpl_dir = 'Tpl';
	
	public function __construct($core,$cache_dir='.epub-files')
	{
		$this->core =& $core;
		$this->cache = $cache_dir;
		$this->images = array();
		$this->files = array();
		$this->dir = array (
			'root' =>  $this->filesPath(),
			'index' => $this->filesPath().'/Index',
			'text' => $this->filesPath().'/Text',
			'images' => $this->filesPath().'/Images',
			'tpl' => $this->filesPath().'/'.$this->tpl_dir
		);
		$this->canWriteCache(true);
		$repository = $this->core->blog->settings->epubmaker->repo;
		if (empty($repository) || !is_writable(path::real($this->core->blog->public_path).'/'.$repository)) {
			$this->repo = $core->blog->public_path;
		} else {
			$this->repo = path::real($this->core->blog->public_path).'/'.$repository;
		}
		
		$this->custom_files = array(
			'styles.css' => $this->core->plugins->moduleRoot('ePUBmaker').'/epub-templates/styles.css',
			'title.html' => $this->core->plugins->moduleRoot('ePUBmaker').'/epub-templates/title.html',
			'cover.html' => $this->core->plugins->moduleRoot('ePUBmaker').'/epub-templates/cover.html'
		);
		
		$this->customFilesExist();
	}
	
	public function filesPath()
	{
		return path::real($this->core->blog->public_path).'/'.$this->cache;
	}
	
	public function filesURL()
	{
		return $this->core->blog->settings->system->public_url.'/'.$this->cache;
	}
	
	public function canWriteCache($create=false)
	{
		$public = path::real($this->core->blog->public_path);
		
		if (!is_dir($public)) {
			$this->core->error->add(__('The \'public\' directory does not exist.'));
			return false;
		}
		
		$this->dir['repo'] = path::real($this->core->blog->public_path).'/'.$this->core->blog->settings->epubmaker->repo;
		
		foreach ($this->dir as $k=>$v) 
		{
			if (!is_dir($v)) {
				if (!is_writable($public)) {
					$this->core->error->add(sprintf(__('The \'%s\' directory cannot be modified.'),'media'));
					unset($this->dir['repo']);
					return false;
				}
				if ($create) {
					files::makeDir($v);
				}
			}
		
			if (!is_writable($v)) {
				$this->core->error->add(sprintf(__('The \'%s\' directory cannot be modified.'),$k));
				unset($this->dir['repo']);
				return false;
			}
		}
		unset($this->dir['repo']);
		return true;
	}
	
	
	public function createEPUB($posts,$title='',$subject='',$filename='')
	{
		try
		{
			if ($filename == '') {
				throw new Exception(__('Empty filename.'));
			}
			
			$filename = files::tidyFileName($filename).'.epub';
			
			if ($title == '') {
				$title = $this->core->blog->name;
			}
			if ($subject == '') {
				$subject = $this->core->blog->desc;
			}
			
			//ob_end_clean();
			while ($posts->fetch() ) {
				$this->createChapter($posts->post_id);
			}
			
			$this->createOpfAndToc($posts,$title,$subject);
			
			$this->registerCover();
			
			$zip_file =  path::real($this->repo).'/'.$filename;
			$fp = fopen($zip_file,'wb');
			$zip = new fileZipFork($fp);
			$zip->addExclusion('#(^|/).(.*?)_(m|s|sq|t).jpg$#');
			$zip->addMimetype();
			$zip->addFile($this->core->plugins->moduleRoot('ePUBmaker').'/src/container.xml','META-INF/container.xml');
			$zip->addFile($this->dir['index'].'/content.opf','OEBPS/content.opf');
			$zip->addFile($this->dir['index'].'/toc.ncx','OEBPS/toc.ncx');
			$zip->addDirectory($this->dir['images'],'OEBPS/Images',true);
			$zip->addDirectory($this->dir['text'],'OEBPS/Text',true);
			$zip->addFile($this->custom_files['cover.jpg'],'OEBPS/Images/cover.jpg');
			$zip->addFile($this->custom_files['styles.css'],'OEBPS/Styles/styles.css');
			//flock($fp, LOCK_EX);
			$zip->write();
			//flock($fp, LOCK_UN);
			$zip->close();
			$this->purgeCache();
		}
		catch (Exception $e)
		{
			@unlink($zip_file);
			throw $e;
		}
		return $zip_file;

	}
	
	public function createChapter($id)
	{
		// _ctx is temporary loaded via ePUBtools::load();
		global $_ctx;
		$params = new ArrayObject(array(
			'post_id' => $id , 
			'post_type' => '')
		);
		
		// Load setting "with images"
		$with_images = $this->core->blog->settings->epubmaker->images;
		
		$_ctx->posts = $this->core->blog->getPosts($params);
		if ($_ctx->posts->post_excerpt_xhtml != '') {
			$trig_excerpt = ePUBtools::triggerImg(
				$_ctx->posts->post_excerpt_xhtml,
				$id,
				$this->dir['images'],
				$with_images
			);
			$_ctx->posts->post_excerpt_xhtml = $trig_excerpt['content'];
			$this->images = array_merge($this->images,$trig_excerpt['images']);
		}
		
		$trig_content = ePUBtools::triggerImg(
			$_ctx->posts->post_content_xhtml,
			$id,
			$this->dir['images'],
			$with_images
		);
		$_ctx->posts->post_content_xhtml = $trig_content['content'];
		$this->images = array_merge($this->images,$trig_content['images']);
		
		$chapter_file = $this->filesPath().'/Text/chapter_'.$id.'.html';
		$fp = fopen($chapter_file,'wb');
		$content = $this->core->tpl->getData('chapter.html');
		fwrite($fp,$content);
		fflush($fp);
		unset($_ctx);
		$this->files[] = $chapter_file;
	}

	public function createOpfAndToc($posts,$title,$subject)
	{
		global $_ctx;
		
		$_ctx->posts = $posts;
		$_ctx->epub_title = $title;
		$_ctx->epub_subject = $subject;
		$_ctx->epub_uuid = ePUBtools::ePUBuuidCreator();
		$_ctx->epub_images = staticRecord::newFromArray($this->images);

		while ($posts->fetch()) {
			$ids[] = $posts->post_id;
		}
		$_ctx->users = ePUBtools::getPostsUsers($ids);
		
		$this->addFile('content.opf','Index');
		$this->addFile('toc.ncx','Index');
		$this->addFile('title.html','Text');
		$this->addFile('cover.html','Text');
		
		unset($_ctx->posts);
	}

	public function addFile($src,$dest)
	{
		try 
		{
			if ($dest!='') {
				$dest = $dest.'/'.$src;
			} else {
				$dest = $src;
			}
			$file = $this->filesPath().'/'.$dest;
			$fp = fopen($file,'wb');
			$content = $this->core->tpl->getData($src);
			fwrite($fp,$content);
			fflush($fp);
			unset($file,$fp,$content);
		}
		catch (Exception $e)
		{
			throw $e;
		}
	}

	public function purgeCache()
	{
		foreach ($this->dir as $k=>$v)
		{
			if (!preg_match('/^tpl|root$/',$k))
			{
				if (is_dir($v)) {
					files::deltree($v);
				}
			}
		}
	}
	
	public function uploadCover($f)
	{
		try
		 {
			$this->media = new dcMedia($this->core);
			$this->media->chdir($this->cache);
			files::uploadStatus($f);
			$this->media->uploadFile($f['tmp_name'],'cover.jpg',null,false,true);
		}
		catch (Exception $e)
		{
			throw $e;
		}
		unset ($this->media);
	}
	
	public function registerCover()
	{
		if (file_exists($this->filesPath().'/cover.jpg')) {
			$this->custom_files['cover.jpg'] = $this->filesPath().'/cover.jpg';
		} else {
			$this->custom_files['cover.jpg'] = $this->core->plugins->moduleRoot('ePUBmaker').'/src/cover.jpg';
		}
	}

	public function deleteCover()
	{
		try
		 {
			$this->media = new dcMedia($this->core);
			$this->media->chdir($this->cache);
			$this->media->removeFile('cover.jpg');
		}
		catch (Exception $e)
		{
			throw $e;
		}
		unset ($this->media);
	}
	
	// Check if filename is editable
	public function checkFile($filename)
	{
		if (array_key_exists($filename,$this->custom_files))
		{
			return true;
		}
		return false;
	}
	
	// Check custom files
	public function customFilesExist()
	{
		foreach($this->custom_files as $k => $v) 
		{
			if (is_file($this->dir['tpl'].'/'.$k) ) 
			{
				$this->custom_files[$k] = $this->dir['tpl'].'/'.$k;
			}
		}
	}
	
	public function isCustomFile($f)
	{
		return is_file($this->dir['tpl'].'/'.$f); 
	}
	
	public function resetFile($f)
	{
		if (!$this->checkFile($f)) {
			throw new Exception(__('File does not exist.'));
		}
		
		if (strpos($this->custom_files[$f],$this->core->plugins->moduleRoot('ePUBmaker').'/epub-templates') !== false) {
			throw new Exception(__('File cannot be removed.'));
		}
				
		if (!files::isDeletable($this->custom_files[$f])) {
			throw new Exception(__('File cannot be removed.'));
		}
		
		if (@unlink($this->custom_files[$f]) === false) {
			throw new Exception(__('File cannot be removed.'));
		}
	}
	
	
	public function getFileContent($f)
	{
		if (!$this->checkFile($f)) {
			throw new Exception(__('File does not exist.'));
		}
		
		$F = $this->custom_files[$f];
		if (!is_readable($F)) {
			throw new Exception(sprintf(__('File %s is not readable'),$f));
		}
		
		return array(
			'c' => file_get_contents($F),
			'w' => $this->getDestinationFile($f) !== false,
			'f' => $f
		);
	}
	
	// Title, cover and style are saved in Tpl dir
	protected function getDestinationFile($f)
	{
		$dest = $this->dir['tpl'].'/'.$f;
		
		if (file_exists($dest) && is_writable($dest)) {
			return $dest;
		}
		
		if (is_writable(dirname($dest))) {
			return $dest;
		} 
		
		return false;
	}
	
	public function writeFile($f,$content)
	{
		try
		{
			$dest = $this->getDestinationFile($f);
			
			if ($dest == false) {
				throw new Exception();
			}

					
			$fp = @fopen($dest,'wb');
			if (!$fp) {
				throw new Exception('tocatch');
			}
			
			$content = preg_replace('/(\r?\n)/m',"\n",$content);
			$content = preg_replace('/\r/m',"\n",$content);
			
			fwrite($fp,$content);
			fclose($fp);
		}
		catch (Exception $e)
		{
			throw new Exception(sprintf(__('Unable to write file %s. Please check your files and folders permissions.'),$f));
		}
	}
}
?>