<?php
# -- BEGIN LICENSE BLOCK ----------------------------------
#
# This file is part of ePUBmaker, a plugin for Dotclear 2.
# 
# Copyright (c) 2012 Osku and contributors
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
#
# -- END LICENSE BLOCK ------------------------------------
if (!defined('DC_RC_PATH')) { return; }

class ePUBtools
{
	public static function checkType($f)
	{
		$name = $f['name'];
		$type = files::getMimeType($name);

		if ($type != 'image/jpeg' && $type != 'image/png') {
			throw new Exception(__('Invalid file type.'));
		}

		return true;
	}
	
	public static function stripTags($content,$with_img=false)
	{
		$allowed_tags = '<h1><h2><h3><h4><h5><h6><p><em><strong><q><cite><blockquote><code><pre><ul><li>';
		
		if ($with_img) {
			$allowed_tags .= '<img>';
		}
		
		$content = strip_tags($content, $allowed_tags);
		return $content;
	}

	public static function triggerImg($content,$post_id,$dirname,$with_img=true)
	{
		global $core;
		
		$content = self::stripTags($content,$with_img);

		$blocks = preg_split('#(<img.*>)#msuU',$content,-1,
			PREG_SPLIT_DELIM_CAPTURE|PREG_SPLIT_NO_EMPTY);
		
		$pu_url = $core->blog->settings->system->public_url;
		$images = array();
		
		foreach ($blocks as $k => $v) {
			if (preg_match('/<img(.*?)\/\>/msu', $v, $matches)) {
				$my_image[$k] = preg_split(
					'#src="(.*?)"#msu', 
					$matches[0],
					-1,
					PREG_SPLIT_DELIM_CAPTURE|PREG_SPLIT_NO_EMPTY
				);
				
				if (strpos($my_image[$k][1],$pu_url) !== false ) {
					// Rename and copy image to working dir
					$src = str_replace($pu_url,'',$my_image[$k][1]);
					$ext = files::getExtension($src);
					$dest = $post_id.'-'.'image-'.$k.'.'.$ext;
					self::imgCopy($src,$dirname.'/'.$dest);
					$images[]['image'] = $dest;
					$my_image[$k][1] = 'src="../Images/'.$dest.'"';
					$blocks[$k] = join($my_image[$k]);
				} else {
					unset($blocks[$k]);
				}
			}
		}
		$content = join($blocks);
		$return = array ('content' => $content, 'images' => $images);
		
		return $return;
	}
	
	public static function imgCopy($src,$dest)
	{
		global $core;
		$src = path::real($core->blog->public_path).'/'.$src;
		copy($src,$dest);
	}
	
	/*
	Return a formatted unique ID for ebook header
	*/
	public static function ePUBuuidCreator()
	{
		$rand = "";
		for ($a = 0; $a < 16; $a++) {
			$rand .= chr(mt_rand(0, 255));
		}
		$uuid = 
			bin2hex(substr($rand,0,4))."-".
			bin2hex(substr($rand,4,2))."-".
			bin2hex(substr($rand,6,2))."-".
			bin2hex(substr($rand,8,2))."-".
			bin2hex(substr($rand,10,6));
		
		return 'urn:uuid:'.$uuid;
	}
	
	/**
	Retrieves all users having posts on current blog.
	
	@param	...
	@return	record
	*/
	public static function getPostsUsers($ids)
	{
		global $core;
		
		$strReq = 'SELECT P.user_id, user_name, user_firstname, '.
				'user_displayname, user_email '.
				'FROM '.$core->prefix.'post P, '.$core->prefix.'user U '.
				'WHERE P.user_id = U.user_id '.
				"AND blog_id = '".$core->con->escape($core->blog->id)."' ".
				'AND P.post_id '.$core->con->in($ids);
		
		$strReq .= 'GROUP BY P.user_id, user_name, user_firstname, user_displayname, user_email ';
		
		return $core->con->select($strReq);
	}
	
	public static function load($output=true)
	{
		global $core;
		
		try 
		{
			$core->ePUBmaker = new ePUBmaker(
				$core,
				$core->blog->settings->epubmaker->dir
			);
			if ($output)
			{
				$_ctx = new context();
				$core->tpl = new dcTemplate(DC_TPL_CACHE,'$core->tpl',$core);
				$core->tpl->setPath(
					$core->ePUBmaker->dir['tpl'],
					dirname(__FILE__).'/../epub-templates',
					$core->tpl->getPath());
				$core->tpl->addBlock('ePUBChapters',array('ePUBtemplate','ePUBChapters'));
				$core->tpl->addValue('ePUBTitle',array('ePUBtemplate','ePUBTitle'));
				$core->tpl->addValue('ePUBSubject',array('ePUBtemplate','ePUBSubject'));
				$core->tpl->addValue('ePUBDate',array('ePUBtemplate','ePUBDate'));
				$core->tpl->addValue('ePUBChapterID',array('ePUBtemplate','ePUBChapterID'));
				$core->tpl->addValue('ePUBplayOrder',array('ePUBtemplate','ePUBplayOrder'));
				$core->tpl->addValue('ePUBUUID',array('ePUBtemplate','ePUBUUID'));
				$core->tpl->addBlock('ePUBContributors',array('ePUBtemplate','ePUBContributors'));
				$core->tpl->addValue('ePUBContributor',array('ePUBtemplate','ePUBContributor'));
				$core->tpl->addValue('ePUBCreator',array('ePUBtemplate','ePUBCreator'));
				$core->tpl->addBlock('ePUBImages',array('ePUBtemplate','ePUBImages'));
				$core->tpl->addValue('ePUBImageName',array('ePUBtemplate','ePUBImageName'));
				$core->tpl->addValue('ePUBImageType',array('ePUBtemplate','ePUBImageType'));
				$core->tpl->addValue('ePUBImageID',array('ePUBtemplate','ePUBImageID'));
			}
		} 
		catch (Exception $e) 
		{
			$core->error->add(__('Someting wrong occurs while loading context and templates.'));
		}
	}
	
	public static function unload()
	{
		unset($_ctx,$core->tpl,$core->ePUBmaker);
	}
}
?>
