<?php
# -- BEGIN LICENSE BLOCK ----------------------------------
#
# This file is part of backupDaemon, a plugin for Dotclear 2.
# 
# Copyright (c) 2012 Osku and contributors
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
#
# -- END LICENSE BLOCK ------------------------------------
if (!defined('DC_CONTEXT_ADMIN')) { return; }
dcPage::check('admin');
$p_name = 'ePUBmaker';
$page_title = __('ePUB maker');

$redir_url = $p_url;

$s =& $core->blog->settings->epubmaker;
$flag		= (boolean)$s->flag;
$images		= (boolean)$s->images;
$repo		= $s->repo;

$file_default = $file = array('c'=>null, 'w'=>false, 'type'=>null, 'f'=>null, 'default_file'=>false);
$reset_button = '';

try
{
	ePUBtools::load(false);
	try
	{
		if (!empty($_REQUEST['file'])) {
			$file = $core->ePUBmaker->getFileContent($_REQUEST['file']);
			$reset_button = $core->ePUBmaker->isCustomFile($_REQUEST['file']) ?  
			'<input class="delete" type="submit" name="reset" value="'.__('Reset').' (r)" accesskey="r" /> ' : '';
		}
	}
	catch (Exception $e)
	{
		$file = $file_default;
		throw $e;
	}
	
	# Save settings
	if (!empty($_POST['saveconfig']))
	{
		$flag		= isset($_POST['flag']) ? true : false;
		$images		= isset($_POST['images']) ? true : false;
		$repo		= html::escapeHTML($_POST['repo']);
		$s->put('flag',$flag,'boolean','ePUBmaker activation flag');
		$s->put('images',$images,'boolean','ePUB with images');
		$s->put('repo',$repo,'string','ePUB files destination directory');
		http::redirect($p_url.'&msg=save');
	}
	
	# Write file
	if (!empty($_POST['write']))
	{
		$file['c'] = $_POST['file_content'];
		$core->ePUBmaker->writeFile($file['f'],$file['c']);
	}
	
	# Reset file
	if (!empty($_POST['reset']))
	{
		$core->ePUBmaker->resetFile($_POST['file']);
		http::redirect($p_url.'&file='.$_POST['file'].'&msg=reset');
	}
	
	if (!empty($_FILES['current_cover']) 
		&& !empty($_FILES['current_cover']['name']) 
		&& ePUBtools::checkType($_FILES['current_cover']))
	{
		$core->ePUBmaker->uploadCover($_FILES['current_cover']);
		http::redirect($p_url.'&msg=cover');
	}

	if (!empty($_POST['delete']))
	{
		$core->ePUBmaker->deleteCover();
		http::redirect($p_url.'&msg=delete');
	}
}
catch (Exception $e)
{
	$core->error->add($e->getMessage());
}

$msg = isset($_REQUEST['msg']) ? $_REQUEST['msg'] : '';
$msg_list = array(
	'save' => __('Configuration has been successfully updated.'),
	'reset' => __('Content has been successfully reset.'),
	'cover' => __('Cover image has been sucessfully uploaded.'),
	'delete' => __('Custom cover image has been successfully removed.'),
	'new' => __('New e-Book ePUB has been created.')
);
if (isset($msg_list[$msg])) {
	$msg = sprintf('<p class="message">%s</p>',$msg_list[$msg]);
}
?>
<html>
<head>
	<title><?php echo $page_title; ?></title>
	<style type="text/css">
		dl dt {
			font-weight: bold;
			font-size: 1.1em;
			margin: 1em 0 0 0;
		}
		dl dd {
			margin: 0 0 1.5em 0;
		}
	</style>
	
  <script type="text/javascript">
  //<![CDATA[
  <?php echo dcPage::jsVar('dotclear.msg.saving_document',__("Saving document...")); ?>
  <?php echo dcPage::jsVar('dotclear.msg.document_saved',__("Document saved")); ?>
  <?php echo dcPage::jsVar('dotclear.msg.error_occurred',__("An error occurred:")); ?>
  //]]>
  </script>
  <script type="text/javascript" src="index.php?pf=ePUBmaker/script.js"></script>
</head>
<body>
<?php
echo $msg;

echo '<h2>'.html::escapeHTML($core->blog->name).' &rsaquo; <span class="page-title">'.$page_title.'</span></h2>';

if ($file['c'] === null)
{
	echo '<div id="epubmaker_options">
	<form method="post" action="'.$redir_url.'">
	<p>'.'<label class="classic" for="flag">'.
	form::checkbox('flag', 1, $flag).__('Enable ePUB creation').
	'</label>
	</p>
	<p>'.'<label class="classic" for="images">'.
	form::checkbox('images', 1, $images).__('Include images').
	'</label>
	</p>
	<p>'.'<label for="repo">'.__('ePUB path:').
	form::field('repo', 30,255, $repo).
	'</label>
	</p>
	<p>'.form::hidden(array('p'),$p_name).
	$core->formNonce().
	'<input type="submit" name="saveconfig" value="'.__('Save').'" /></p>
	</form></div>
	<div class="two-cols"><div class="col">
	<dl class="modules">
		<dt><a href="'.$p_url.'&file=styles.css">'.__('Edit stylesheet').'</a></dt>
		<dd>'.__('Change stylesheet included with the ePUB file.').'</dd>
		<dt><a href="'.$p_url.'&file=title.html">'.__('Edit title').'</a></dt>
		<dd>'.__('Change ePUB title template.').'</dd>	
		<dt><a href="'.$p_url.'&file=cover.html">'.__('Edit cover').'</a></dt>
		<dd>'.__('Change ePUB cover template.').'</dd>
	</dl>
	
	</div><div class="col">';
	echo '<h3>'.__('Cover image').'</h3>';
	echo '
		<form method="post" action="'.$p_url.'" enctype="multipart/form-data">
		<p id="uploader" class="field"><label for="current_cover">'.__('New cover:').'</label>'.
		'<input type="file" name="current_cover" id="current_cover" size="15" />'.
		'</p>
		<p class="form-note">'.__('Recommanded size: 500×800px.').'</p>
		<p><input type="submit" name="upload" value="'.__('Send').'" /> '.
		$core->formNonce().
		'</p>
		</form>';
	if (file_exists($core->ePUBmaker->filesPath().'/cover.jpg')) {
		echo '<form method="post" action="'.$p_url.'">
			<h4>'.__('Current image').'</h4>
			<img 
			src="'.$core->ePUBmaker->filesURL().'/.cover_sq.jpg" 
			title="'.__('Current cover').'"
			alt="'.html::escapeHTML(__('Current cover')).'" />
			<p><input class="delete" type="submit" name="delete" value="'.__('Delete').'" /></p>'.
			$core->formNonce().
			'</form>';
	}
	echo '</div></div>';
}
else
{
	echo
	'<p><a class="back" href="'.$p_url.'">'.__('Back').'</a></p>
	<div id="file-editor"><form id="file-form" action="'.$p_url.'" method="post">'.
	'<p><label for="file_content">'.sprintf(__('Editing file %s'),'<strong>'.$file['f']).'</strong></label></p>'.
	'<p>'.form::textarea('file_content',50,25,html::escapeHTML($file['c']),'maximal','',!$file['w']).'</p>';
	
	if ($file['w'])
	{
		echo
		'<p><input type="submit" name="write" value="'.__('Save').' (s)" accesskey="s" /> '.
		$reset_button.
		$core->formNonce().
		form::hidden(array('file'),$_REQUEST['file']).
		'</p>';
	}
	else
	{
		echo '<p>'.__('This file is not writable. Please check your theme files permissions.').'</p>';
	}
	
	echo
	'</form></div>';
}
?>
</body>
</html>